﻿using ff14bot;
using ff14bot.Managers;
using System.Linq;
using System.Threading.Tasks;
using UltimaCR.Spells.Main;

namespace UltimaCR.Rotations
{
    public sealed partial class Paladin
    {
        private PaladinSpells _mySpells;

        private PaladinSpells MySpells
        {
            get { return _mySpells ?? (_mySpells = new PaladinSpells()); }
        }

        #region Class Spells

        private async Task<bool> FastBlade()
        {
            return await MySpells.FastBlade.Cast();
        }

        private async Task<bool> Rampart()
        {
            return await MySpells.Rampart.Cast();
        }

        private async Task<bool> RiotBlade()
        {
            if (Ultima.UltSettings.SingleTarget &&
		Actionmanager.LastSpell.Name == MySpells.FastBlade.Name &&
		Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth * .35 &&
		Core.Player.CurrentTarget.HasAura(724, false, 7000) ||

		!Ultima.UltSettings.SingleTarget &&
		Actionmanager.LastSpell.Name == MySpells.FastBlade.Name &&
		!Actionmanager.HasSpell(MySpells.RoyalAuthority.Name) ||

		!Ultima.UltSettings.SingleTarget &&
		Actionmanager.LastSpell.Name == MySpells.FastBlade.Name &&
		Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth * .35 &&
		Actionmanager.HasSpell(MySpells.RoyalAuthority.Name))
            {
                if (Actionmanager.HasSpell(MySpells.GoringBlade.Name) &&
		    !Core.Player.CurrentTarget.HasAura(MySpells.GoringBlade.Name, true, 8000))
                {
                    return await MySpells.RiotBlade.Cast();
                }
            }
            return false;
        }

        private async Task<bool> SavageBlade()
        {
            if (Actionmanager.LastSpell.Name == MySpells.FastBlade.Name)
            {
                return await MySpells.SavageBlade.Cast();
            }
            return false;
        }

        private async Task<bool> OPENERFightOrFlight()
        {
            if (!Core.Player.InCombat &&
		Core.Player.HasTarget &&
		Core.Player.CurrentTarget.CurrentHealthPercent >= 99)
            {
                if (Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth * 4)
                {
                    return await MySpells.FightOrFlight.Cast();
		}
            }
            return false;
        }

        private async Task<bool> FightOrFlight()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldLob.Name &&
		Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name &&
		Core.Player.InCombat &&
		Core.Player.HasTarget &&
		Core.Player.TargetDistance(4, false))
            {
                if (Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth  ||

		    Core.Player.CurrentTarget.CurrentHealth <= Core.Player.MaxHealth &&
		    Core.Player.CurrentTarget.CurrentHealthPercent > 70)
                {
                    return await MySpells.FightOrFlight.Cast();
		}
            }
            return false;
        }

        private async Task<bool> Flash()
        {
            if (Actionmanager.LastSpell.Name == MySpells.SavageBlade.Name &&
		Helpers.EnemiesNearPlayer(5) > 1 ||

		Actionmanager.LastSpell.Name == MySpells.RiotBlade.Name &&
		Helpers.EnemiesNearPlayer(5) > 1 ||

		!Actionmanager.HasSpell(MySpells.CircleOfScorn.Name) &&
		Ultima.LastSpell.Name == MySpells.ShieldLob.Name &&
		Helpers.EnemiesNearPlayer(5) > 1 ||

		Actionmanager.HasSpell(MySpells.CircleOfScorn.Name) &&
		Ultima.LastSpell.Name == MySpells.CircleOfScorn.Name &&
		Helpers.EnemiesNearPlayer(5) > 1)
            {
                return await MySpells.Flash.Cast();
            }
            return false;
        }

        private async Task<bool> Convalescence()
        {
            return await MySpells.Convalescence.Cast();
        }

        private async Task<bool> ShieldLob()
        {
            if (Core.Player.TargetDistance(10) &&
		!Actionmanager.HasSpell(MySpells.ShieldOath.Name) ||

		Core.Player.TargetDistance(10) &&
		Actionmanager.HasSpell(MySpells.ShieldOath.Name) &&
		Core.Player.HasAura(MySpells.ShieldOath.Name))
            {
                if (!Core.Player.InCombat &&
		    !Actionmanager.CanCast(MySpells.FightOrFlight.Name, Core.Player) ||

		    Core.Player.InCombat &&
		    Ultima.LastSpell.Name != MySpells.ShieldLob.Name &&
		    Core.Player.CurrentTarget.CurrentHealthPercent >= 99)
                {
                    return await MySpells.ShieldLob.Cast();
		}
            }
            return false;
        }

        private async Task<bool> ShieldBash()
        {
            return await MySpells.ShieldBash.Cast();
        }

        private async Task<bool> Provoke()
        {
            var target = Helpers.NotTargetingPlayer.FirstOrDefault();

            if (target != null)
            {
                return await MySpells.Provoke.Cast(target);
            }
            return false;
        }

        private async Task<bool> RageOfHalone()
        {
            if (Actionmanager.LastSpell.Name == MySpells.SavageBlade.Name)
            {
                if (!Actionmanager.HasSpell(MySpells.RoyalAuthority.Name) ||

		    !Ultima.UltSettings.MultiTarget &&
		    Actionmanager.HasSpell(MySpells.RoyalAuthority.Name) &&
		    Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth * .35)
                {
                    return await MySpells.RageOfHalone.Cast();
                }
            }
            return false;
        }

        private async Task<bool> GoringBlade()
        {
            if (Actionmanager.LastSpell.Name == MySpells.RiotBlade.Name)
            {
                if (!Core.Player.CurrentTarget.HasAura(MySpells.GoringBlade.Name, true, 8000) ||

		    Actionmanager.LastSpell.Name == MySpells.SavageBlade.Name)
                {
                    return await MySpells.GoringBlade.Cast();
                }
            }
            return false;
        }

        private async Task<bool> RoyalAuthority()
        {
            if (Actionmanager.LastSpell.Name == MySpells.SavageBlade.Name)
            {
                if (!Ultima.UltSettings.MultiTarget &&
		    Core.Player.CurrentTarget.HasAura(724, false, 7000) &&
		    Core.Player.CurrentTarget.HasAura(MySpells.GoringBlade.Name, true) ||

		    Ultima.UltSettings.MultiTarget ||

		    Core.Player.CurrentTarget.CurrentHealth <= Core.Player.MaxHealth * .35)
                {
                    return await MySpells.RoyalAuthority.Cast();
                }
            }
            return false;
        }

        private async Task<bool> ShieldSwipe()
        {
            if (Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name)
            {
                if (Ultima.UltSettings.PaladinShieldSwipe)
                {
                    return await MySpells.ShieldSwipe.Cast();
		}
            }
            return false;
        }

        private async Task<bool> Awareness()
        {
            return await MySpells.Awareness.Cast();
        }

        private async Task<bool> Sentinel()
        {
            return await MySpells.Sentinel.Cast();
        }

        private async Task<bool> TemperedWill()
        {
            return await MySpells.TemperedWill.Cast();
        }

        private async Task<bool> Bulwark()
        {
            return await MySpells.Bulwark.Cast();
        }

        private async Task<bool> CircleOfScorn()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name &&
		Core.Player.InCombat &&
		Core.Player.HasTarget && 
		Core.Player.CurrentTarget.CurrentHealth < Core.Player.MaxHealth / 2 &&
		Core.Player.CurrentTarget.CurrentHealthPercent > 50 ||

		Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name &&
		Core.Player.InCombat &&
		Core.Player.HasTarget && 
		Core.Player.CurrentTarget.CurrentHealth >= Core.Player.MaxHealth / 2)
            {
                if (Ultima.UltSettings.SingleTarget &&
		    Ultima.LastSpell.Name != MySpells.ShieldLob.Name &&
		    !Actionmanager.CanCast(MySpells.FightOrFlight.Name, Core.Player) &&
		    Core.Player.TargetDistance(3, false) ||

		    Ultima.UltSettings.SmartTarget &&
		    Ultima.LastSpell.Name != MySpells.ShieldLob.Name &&
		    !Actionmanager.CanCast(MySpells.FightOrFlight.Name, Core.Player) &&
		    Helpers.EnemiesNearPlayer(5) > 1 ||

		    Ultima.UltSettings.SingleTarget &&
		    Ultima.LastSpell.Name == MySpells.ShieldLob.Name &&
		    Core.Player.TargetDistance(3, false) ||

		    Ultima.UltSettings.SmartTarget &&
		    Ultima.LastSpell.Name == MySpells.ShieldLob.Name &&
		    Helpers.EnemiesNearPlayer(5) > 1 ||

		    Ultima.UltSettings.MultiTarget &&
		    !Actionmanager.CanCast(MySpells.FightOrFlight.Name, Core.Player) &&
		    Core.Player.CurrentTarget.MaxHealth > Core.Player.MaxHealth * 10 &&
		    Core.Player.TargetDistance(3, false))
                {
                    return await MySpells.CircleOfScorn.Cast();
		}
            }
            return false;
        }


        #endregion

        #region Cross Class Spells

        #region Conjurer

        private async Task<bool> Cure()
        {
            if (Ultima.UltSettings.PaladinCure)
            {
                return await MySpells.CrossClass.Cure.Cast();
            }
            return false;
        }

        private async Task<bool> Protect()
        {
            if (Ultima.UltSettings.PaladinProtect &&
		!Core.Player.InCombat &&
                !Core.Player.HasAura(MySpells.CrossClass.Protect.Name))
            {
                return await MySpells.CrossClass.Protect.Cast();
            }
            return false;
        }

        private async Task<bool> Raise()
        {
            if (Ultima.UltSettings.PaladinRaise)
            {
                return await MySpells.CrossClass.Raise.Cast();
            }
            return false;
        }

        private async Task<bool> Stoneskin()
        {
            if (Ultima.UltSettings.PaladinStoneskin &&
		!Core.Player.InCombat &&
                !Core.Player.HasAura(MySpells.CrossClass.Stoneskin.Name))
            {
                return await MySpells.CrossClass.Stoneskin.Cast();
            }
            return false;
        }

        #endregion

        #region Marauder

        private async Task<bool> Foresight()
        {
            if (Ultima.UltSettings.PaladinForesight)
            {
                return await MySpells.CrossClass.Foresight.Cast();
            }
            return false;
        }

        private async Task<bool> SkullSunder()
        {
            if (Ultima.UltSettings.PaladinSkullSunder)
            {
                return await MySpells.CrossClass.SkullSunder.Cast();
            }
            return false;
        }

        private async Task<bool> Fracture()
        {
            if (Ultima.UltSettings.PaladinFracture &&
                !Core.Player.CurrentTarget.HasAura(MySpells.CrossClass.Fracture.Name, true, 4000))
            {
                return await MySpells.CrossClass.Fracture.Cast();
            }
            return false;
        }

        private async Task<bool> Bloodbath()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name)
            {
                if (Core.Player.HasTarget &&
		    Core.Player.InCombat &&
		    Core.Player.TargetDistance(4, false) &&
		    Core.Player.CurrentHealthPercent <= 60)
                {
                    return await MySpells.CrossClass.Bloodbath.Cast();
		}
            }
            return false;
        }

        private async Task<bool> MercyStroke()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name)
            {
                if (Ultima.UltSettings.PaladinMercyStroke)
                {
                    return await MySpells.CrossClass.MercyStroke.Cast();
		}
            }
            return false;
        }

        #endregion

        #endregion

        #region Job Spells

        private async Task<bool> SwordOath()
        {
            if (Ultima.UltSettings.PaladinSwordOath &&
		!Core.Player.HasAura(MySpells.SwordOath.Name) &&
		!Core.Player.HasAura(MySpells.ShieldOath.Name) ||
                !Actionmanager.HasSpell(MySpells.ShieldOath.Name) &&
		!Core.Player.HasAura(MySpells.SwordOath.Name) &&
		!Core.Player.HasAura(MySpells.ShieldOath.Name))
            {
                if (Core.Player.CurrentHealthPercent >= 89 &&
		    Core.Player.HasTarget &&
		    Core.Player.CurrentTarget.CanAttack &&
		    !Core.Player.InCombat ||
		    Core.Player.InCombat &&
		    Core.Player.HasTarget)
                {
                    return await MySpells.SwordOath.Cast();
                }
            }
            return false;
        }

        private async Task<bool> Cover()
        {
            return await MySpells.Cover.Cast();
        }

        private async Task<bool> ShieldOath()
        {
            if (Ultima.UltSettings.PaladinShieldOath &&
		!Core.Player.HasAura(MySpells.SwordOath.Name) &&
		!Core.Player.HasAura(MySpells.ShieldOath.Name))
            {
                if (Core.Player.CurrentHealthPercent >= 89 &&
		    Core.Player.HasTarget &&
		    !Core.Player.InCombat ||
		    Core.Player.InCombat &&
		    Core.Player.HasTarget)
                {
                    return await MySpells.ShieldOath.Cast();
                }
            }
            return false;
        }

        private async Task<bool> SpiritsWithin()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.Sheltron.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name)
            {
                if (Ultima.UltSettings.PaladinSpiritsWithin &&
		    !Actionmanager.CanCast(MySpells.FightOrFlight.Name, Core.Player))
                {
                    return await MySpells.SpiritsWithin.Cast();
		}
            }
            return false;
        }

        private async Task<bool> HallowedGround()
        {
            if (!Ultima.UltSettings.SingleTarget &&
		Core.Player.InCombat &&
		Core.Player.CurrentHealthPercent <= 19)
            {
                return await MySpells.HallowedGround.Cast();
            }
            return false;
        }

        private async Task<bool> Sheltron()
        {
            if (Ultima.LastSpell.Name != MySpells.ShieldSwipe.Name &&
		Ultima.LastSpell.Name != MySpells.SpiritsWithin.Name &&
		Ultima.LastSpell.Name != MySpells.FightOrFlight.Name &&
		Ultima.LastSpell.Name != MySpells.CircleOfScorn.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.Bloodbath.Name &&
		Ultima.LastSpell.Name != MySpells.CrossClass.MercyStroke.Name &&
		Core.Player.HasTarget &&
		Core.Player.InCombat &&
                Core.Player.CurrentTarget.IsFacing(Core.Player)&&
		Core.Player.TargetDistance(3, false))
            {
                if (!Ultima.UltSettings.SingleTarget &&
		    Core.Player.CurrentTarget.CurrentHealth > Core.Player.MaxHealth ||
		
		    !Ultima.UltSettings.SingleTarget &&
		    Core.Player.CurrentTarget.CurrentHealth <= Core.Player.MaxHealth &&
		    Core.Player.CurrentTarget.CurrentHealthPercent > 50)
                {
                    return await MySpells.Sheltron.Cast();
		}
            }
            return false;
        }

        private async Task<bool> DivineVeil()
        {
            return await MySpells.DivineVeil.Cast();
        }

        private async Task<bool> Clemency()
        {
            if (Actionmanager.LastSpell.Name == MySpells.RageOfHalone.Name &&
		Core.Player.HasAura(79) &&
		Core.Player.CurrentHealthPercent < 35 ||
		Actionmanager.LastSpell.Name == MySpells.GoringBlade.Name &&
		Core.Player.HasAura(79) &&
		Core.Player.CurrentHealthPercent < 35 ||
		Actionmanager.LastSpell.Name == MySpells.RoyalAuthority.Name &&
		Core.Player.HasAura(79) &&
		Core.Player.CurrentHealthPercent < 35)
            {
                return await MySpells.Clemency.Cast();
            }
            return false;
        }

        #endregion

        #region PvP Spells

        private async Task<bool> Enliven()
        {
            return await MySpells.PvP.Enliven.Cast();
        }

        private async Task<bool> FullSwing()
        {
            return await MySpells.PvP.FullSwing.Cast();
        }

        private async Task<bool> GlorySlash()
        {
            return await MySpells.PvP.GlorySlash.Cast();
        }

        private async Task<bool> Purify()
        {
            return await MySpells.PvP.Purify.Cast();
        }

        private async Task<bool> Testudo()
        {
            return await MySpells.PvP.Testudo.Cast();
        }

        #endregion
    }
}